import os
import subprocess
import unittest


class TestFlake8Clean(unittest.TestCase):
    """ensure that the tree is flake8 clean"""

    def test_flake8_clean(self):
        top_src_dir = os.path.join(os.path.dirname(__file__), "..")
        targets = [
            top_src_dir,
            os.path.join(top_src_dir, "unattended-upgrade"),
            os.path.join(top_src_dir, "unattended-upgrade-shutdown"),
        ]
        subprocess.check_call(
            [
                "flake8",
                # use max-line-length=88 for upcoming "black" compatibility
                "--max-line-length=88",
                # E402 ignored because we need to import/config apt_pkg before
                #      importing unattended_upgrade
                # W503 ignored because some lines with binary operators are long
                "--ignore=E402,W503",
            ]
            + targets
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

import logging
import os
import os.path
import unittest

import apt_pkg

from test.test_base import TestBase, MockOptions

from unattended_upgrade import _setup_logging


class TestLogInstallProgress(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.mock_options = MockOptions()

    def test_log_installprogress(self):
        logdir = os.path.join(self.tempdir, "mylog")
        apt_pkg.config.set("Unattended-Upgrade::LogDir", logdir)
        # FIXME: this test is really not testing much, see if that
        # can be improved
        _setup_logging(self.mock_options)
        self.assertTrue(os.path.exists(logdir))


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
"""Test unattended_upgrades against the real archive in a chroot.

Note that this test is not run by the makefile in this folder, as it requires
network access, and it fails in some situations (unclear which).
"""

import glob
import logging
import os
import re
import unittest

import apt_pkg
import apt

import unattended_upgrade
from test.test_base import TestBase, MockOptions


apt_pkg.config.set("APT::Architecture", "amd64")


# FIXME: port to something more recent than lucid(!)
class TestAgainstRealArchive(TestBase):
    @unittest.skip("FIXME: test needs porting")
    @unittest.skipIf(os.getuid() != 0, "must run as root")
    def setUp(self):
        TestBase.setUp(self)
        for g in ["./aptroot/var/log/apt/*", "./aptroot/var/log/*"]:
            for f in glob.glob(g):
                if os.path.isfile(f):
                    os.remove(f)
        # get a lucid based cache (test good until 04/2015)
        cache = apt.Cache(rootdir="./aptroot")
        cache.update()
        del cache
        # ensure apt does not do any post-invoke stuff that fails
        # (because we are not root)
        apt_pkg.config.clear("DPkg::Post-Invoke")
        apt_pkg.config.clear("DPkg::Pre-Invoke")
        unattended_upgrade.DISTRO_CODENAME = "lucid"

    def test_against_real_archive(self):
        # create mock options
        options = MockOptions(dry_run=False, debug=True)
        # run unattended-upgrades against fake system
        logdir = os.path.abspath("./aptroot/var/log/")
        logfile = os.path.join(logdir, "unattended-upgrades.log")
        apt_pkg.config.set("APT::UnattendedUpgrades::LogDir", logdir)

        # main
        res = unattended_upgrade.main(options, os.path.abspath("./aptroot"))
        logging.debug(res)
        # check if the log file exists
        self.assertTrue(os.path.exists(logfile))
        with open(logfile) as fp:
            log = fp.read()
        # check that stuff worked
        self.assertFalse(" ERROR " in log, log)
        # check if we actually have the expected ugprade in it
        self.assertTrue(re.search("INFO Packages that will be upgraded:.*awstats", log))
        # apt-doc has a higher version in -updates than in -security
        # and no other dependencies so its a perfect test
        self.assertTrue(re.search("INFO Packages that will be upgraded:.*apt-doc", log))
        self.assertFalse(
            re.search("INFO Packages that will be upgraded:.*ant-doc", log)
        )
        self.assertTrue(re.search("DEBUG skipping blacklisted package 'ant-doc'", log))
        # test dpkg install log
        # term_log = open("aptroot/var/log/apt/term.log").read()
        # FIXME: when we redirect STDIN the below test will break - however
        #        we need to redirect it as otherwise we may hang forever
        #        - this is actually a bug in apt that uses "tcgetattr(0, &tt)"
        #          on FD=0 instead of FD=1
        # print term_log
        # self.assertTrue(
        #    re.search(
        #        "fake-dpkg: --status-fd .* --configure.*awstats", term_log))


if __name__ == "__main__":
    import locale

    locale.setlocale(locale.LC_ALL, "C")
    unittest.main()



#!/usr/bin/python3

import os
import unittest

from test.test_base import TestBase


class TestUntrusted(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.rootdir = self.make_fake_aptroot(
            template=os.path.join(self.testdir, "root.untrusted"))
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from email.parser import Parser
from io import StringIO
from unittest.mock import patch
import os
from textwrap import dedent
import unittest


import apt_pkg

import unattended_upgrade
from unattended_upgrade import (
    get_dpkg_log_content,
    LoggingDateTime,
    send_summary_mail,
    setup_apt_listchanges,
)
from test.test_base import TestBase, MockOptions


class ExtractDpkgLogTestCase(TestBase):

    def test_get_dpkg_log_content(self):
        logfile_dpkg = os.path.join(self.tempdir, "apt-term.log")
        # note that we intentionally not have a "Log ended:" here
        # because this may happen if something crashes power goes
        # down etc
        OLD_LOG = dedent("""\
            Log started: 2013-01-01  12:00:00
            old logfile text
        """)
        NEW_LOG = dedent("""\
            Log started: 2014-10-28  10:00:00
            random logfile_dpkg text
            Log ended: 2013-01-01  12:20:00

            Log started: 2014-10-28  12:21:00
            more random logfile_dpkg text
            Log ended: 2013-01-01  12:30:00
            """)
        with open(logfile_dpkg, "w") as fp:
            fp.write(OLD_LOG)
            fp.write("\n")
            fp.write(NEW_LOG)
        start_time = LoggingDateTime.from_string("2014-10-28  10:00:00")
        dpkg_log_content = get_dpkg_log_content(logfile_dpkg, start_time)
        self.assertEqual(dpkg_log_content, NEW_LOG)


# note this is not a unittest.TestCase as it needs to be parameterized
class CommonTestsForMailxAndSendmail(object):

    EXPECTED_MAIL_CONTENT_STRINGS = [
        "random logfile_dpkg text",
        "mem_log text",
    ]
    NOT_EXPECTED_MAIL_CONTENT_STRINGS = [
        "old logfile text",
    ]

    def common_setup(self):
        # monkey patch to make it testable
        unattended_upgrade.REBOOT_REQUIRED_FILE = os.path.join(
            self.tempdir, "reboot-required")
        unattended_upgrade.MAIL_BINARY = "./no-mailx-binary-here"
        unattended_upgrade.SENDMAIL_BINARY = "./no-sendmail-binary-here"
        # setup mail
        apt_pkg.config.set("Unattended-Upgrade::Mail", "root")
        apt_pkg.config.set("Unattended-Upgrade::MailReport", "on-change")
        apt_pkg.config.set("Unattended-Upgrade::LogDir", self.tempdir)

    def _return_mock_data(self, successful=True):
        """ return input tuple for send_summary_mail """
        pkgs = ["2vcard"]
        res = successful
        result_str = "Result String"
        pkgs_kept_back = {"Debian wheezy-security": ["linux-image"]}
        pkgs_removed = ["telnet"]
        pkgs_kept_installed = ["hello"]
        # include some unicode chars here for good measure
        mem_log = StringIO("""mem_log text üöä
Allowed origins are: ['o=Debian,n=wheezy', 'o=Debian,n=wheezy-updates',\
 'o=Debian,n=wheezy,l=Debian-Security', 'origin=Debian,archive=stable,label=\
Debian-Security']
""")
        dpkg_log_content = dedent("""\
        Log started: 2014-10-28  12:21:00
        random logfile_dpkg text
        Log ended: 2013-01-01  12:30:00
        """)
        return (pkgs, res, result_str, pkgs_kept_back, pkgs_removed,
                pkgs_kept_installed, mem_log, dpkg_log_content)

    def _verify_common_mail_content(self, mail_txt):
        for expected_string in self.EXPECTED_MAIL_CONTENT_STRINGS:
            self.assertTrue(expected_string in mail_txt)
        for not_expected_string in self.NOT_EXPECTED_MAIL_CONTENT_STRINGS:
            self.assertFalse(not_expected_string in mail_txt)
        self.assertEqual(mail_txt.count("Log started: "), 1)

    def test_summary_mail_reboot(self):
        with open(unattended_upgrade.REBOOT_REQUIRED_FILE, "w") as fp:
            fp.write("")
        send_summary_mail(*self._return_mock_data())
        # this is used for py2 compat for py3 only we can do
        # remove the "rb" and the subsequent '.decode("utf-8")'
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue("[reboot required]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue("Packages that were upgraded:\n 2vcard" in mail_txt)

    def test_summary_mail_no_reboot(self):
        send_summary_mail(*self._return_mock_data())
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertFalse("[reboot required]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue("Packages that were upgraded:\n 2vcard" in mail_txt)

    def test_summary_mail_only_on_error(self):
        # default is to always send mail, ensure this is correct
        # for both success and failure
        apt_pkg.config.set("Unattended-Upgrade::MailReport", "on-change")
        send_summary_mail(*self._return_mock_data(successful=True))
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            self._verify_common_mail_content(fp.read().decode("utf-8"))
        os.remove(os.path.join(self.tempdir, "mail.txt"))
        # now with a simulated failure
        send_summary_mail(*self._return_mock_data(successful=False))
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            self._verify_common_mail_content(fp.read().decode("utf-8"))
        os.remove(os.path.join(self.tempdir, "mail.txt"))
        # now test with "only-on-error"
        apt_pkg.config.set("Unattended-Upgrade::MailReport", "only-on-error")
        send_summary_mail(*self._return_mock_data(successful=True))
        self.assertFalse(
            os.path.exists(os.path.join(self.tempdir, "mail.txt")))
        send_summary_mail(*self._return_mock_data(successful=False))
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self._verify_common_mail_content(mail_txt)
        self.assertTrue("Unattended upgrade result: Result String" in mail_txt)
        self.assertTrue(
            os.path.exists(os.path.join(self.tempdir, "mail.txt")))
        self.assertTrue(
            "Packages that attempted to upgrade:\n 2vcard" in mail_txt)

    def test_summary_mail_blacklisted(self):
        # Test that blacklisted packages are mentioned in the mail message.
        send_summary_mail(*self._return_mock_data())
        self.assertTrue(
            os.path.exists(os.path.join(self.tempdir, "mail.txt")))
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue("[package on hold]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue(
            "Packages with upgradable origin but kept back:\n"
            " Debian wheezy-security:\n  linux-image"
            in mail_txt)

    def test_summary_mail_blacklisted_only(self):
        # Test that when only blacklisted packages are available, they
        # are still mentioned in the mail message.
        pkgs, res, result_str, pkgs_kept_back, pkgs_removed, \
            pkgs_kept_installed, mem_log, logf_dpkg = self._return_mock_data(
                successful=True)
        pkgs = []
        send_summary_mail(pkgs, res, result_str, pkgs_kept_back, pkgs_removed,
                          pkgs_kept_installed, mem_log, logf_dpkg)
        self.assertTrue(
            os.path.exists(os.path.join(self.tempdir, "mail.txt")))
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue("[package on hold]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue(
            "Packages with upgradable origin but kept back:\n"
            " Debian wheezy-security:\n  linux-image"
            in mail_txt)
        self.assertFalse(
            "Packages that attempted to upgrade:\n 2vcard" in mail_txt)

    def test_apt_listchanges(self):
        # test with sendmail available
        unattended_upgrade.SENDMAIL_BINARY = "/bin/true"
        setup_apt_listchanges("./data/listchanges.conf.mail")
        self.assertEqual(os.environ["APT_LISTCHANGES_FRONTEND"], "mail")
        # test without sendmail
        unattended_upgrade.SENDMAIL_BINARY = "/bin/not-here-xxxxxxxxx"
        setup_apt_listchanges("./data/listchanges.conf.pager")
        self.assertEqual(os.environ["APT_LISTCHANGES_FRONTEND"], "none")

    def test_summary_mail_from_address(self):
        apt_pkg.config.set("Unattended-Upgrade::Sender", "rootolv")
        self.addCleanup(apt_pkg.config.set, "Unattended-Upgrade::Sender", "")
        send_summary_mail(*self._return_mock_data())
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue(
            "From: rootolv" in mail_txt, "missing From: in %s" % mail_txt)

    @patch("unattended_upgrade.run")
    def test_exception(self, mock_run):
        exception_string = "Test exception for email"
        mock_run.side_effect = Exception(exception_string)
        # run it
        options = MockOptions()
        exception_raised = False
        try:
            unattended_upgrade.main(options)
        except Exception as e:
            self.assertEqual(str(e), exception_string)
            exception_raised = True
        self.assertTrue(exception_raised)
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue(exception_string in mail_txt)


class MailxTestCase(CommonTestsForMailxAndSendmail, TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.common_setup()
        unattended_upgrade.MAIL_BINARY = make_mock_mailx(self.tempdir)

    def _verify_common_mail_content(self, mail_txt):
        CommonTestsForMailxAndSendmail._verify_common_mail_content(
            self, mail_txt)
        # setting this header with mailx is not possible so ensure
        # we don't accidently try
        self.assertFalse('text/plain; charset="utf-8"' in mail_txt)


def make_mock_sendmail(tmpdir):
    p = os.path.join(tmpdir, "mock-sendmail")
    with open(p, "w") as fp:
        fp.write("""#!/bin/sh
cat - -- > %s/mail.txt
    """ % tmpdir)
    os.chmod(p, 0o755)
    return p


def make_mock_mailx(tmpdir):
    p = os.path.join(tmpdir, "mock-mail")
    with open(p, "w") as fp:
        fp.write("""#!/bin/sh

echo "From: $2" > %(tmp)s/mail.txt
echo "Subject: $4" >> %(tmp)s/mail.txt
cat - -- >> %(tmp)s/mail.txt
        """ % {'tmp': tmpdir})
    os.chmod(p, 0o755)
    return p


class SendmailTestCase(CommonTestsForMailxAndSendmail, TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.common_setup()
        unattended_upgrade.SENDMAIL_BINARY = make_mock_sendmail(self.tempdir)

    def _verify_common_mail_content(self, mail_txt):
        CommonTestsForMailxAndSendmail._verify_common_mail_content(
            self, mail_txt)

        msg = Parser().parsestr(mail_txt)
        content_type = msg["Content-Type"]
        self.assertEqual(content_type, 'text/plain; charset="utf-8"')

    def test_mail_quoted_printable(self):
        """Regression test for debian bug #700178"""
        send_summary_mail(*self._return_mock_data())
        with open(os.path.join(self.tempdir, "mail.txt"), "rb") as fp:
            log_data = fp.read().decode("utf-8")
        needle = "Allowed origins are: ['o=3DDebian,n=3Dwheezy', "\
            "'o=3DDebian,n=3Dwheezy-updat=\n"\
            "es', 'o=3DDebian,n=3Dwheezy,l=3DDebian-Security', "\
            "'origin=3DDebian,archive=\n"\
            "=3Dstable,label=3DDebian-Security']"
        self.assertTrue(needle in log_data)


class SendmailAndMailxTestCase(SendmailTestCase):

    def setUp(self):
        SendmailTestCase.setUp(self)
        self.common_setup()
        unattended_upgrade.MAIL_BINARY = make_mock_mailx(self.tempdir)
        unattended_upgrade.SENDMAIL_BINARY = make_mock_sendmail(self.tempdir)


if __name__ == "__main__":
    # logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import dbusmock
from gi.repository import Gio, GLib

import os
import subprocess
import threading
import unittest

import apt

from test.test_base import TestBase, MockOptions
import unattended_upgrade

apt.apt_pkg.config.set("APT::Architecture", "amd64")

APT_CONF = """
Unattended-Upgrade::Keep-Debs-After-Install "true";
Unattended-Upgrade::Allowed-Origins {{
    "Ubuntu:lucid-security";
}};
Unattended-Upgrade::Skip-Updates-On-Metered-Connections "false";
Unattended-Upgrade::Postpone-For-Days "{days}";
Unattended-Upgrade::Postpone-Wait-Time "2";
"""


class PostponeClient(threading.Thread):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.on_about_to_start = None
        self.on_started = None
        self.on_finished = None
        self.on_canceled = None
        self.received = []
        self.error = None

        self.proxy = None
        self.loop = None

    def call_postpone(self):
        def on_res(proxy, res):
            try:
                proxy.call_finish(res)
            except GLib.GError as e:
                self.error = e

        if self.proxy:
            self.proxy.call(
                "Postpone", None,
                Gio.DBusCallFlags.ALLOW_INTERACTIVE_AUTHORIZATION,
                -1, None, on_res)

    def quit(self):
        if self.loop:
            self.loop.quit()

    def start(self):
        self.started_sem = threading.Semaphore(0)
        threading.Thread.start(self)
        self.started_sem.acquire(timeout=10)

    def run(self):
        self.thread_context = GLib.MainContext()
        self.thread_context.push_thread_default()
        self.loop = GLib.MainLoop(self.thread_context)
        self.proxy = Gio.DBusProxy.new_for_bus_sync(
            Gio.BusType.SYSTEM,
            Gio.DBusProxyFlags.NONE,
            None,
            "com.ubuntu.UnattendedUpgrade",
            "/com/ubuntu/UnattendedUpgrade/Pending",
            "com.ubuntu.UnattendedUpgrade.Pending",
            None)

        def on_signal(proxy, sender, signal, params):
            self.received.append(signal)
            if signal == "AboutToStart":
                if self.on_about_to_start:
                    self.on_about_to_start()
            elif signal == "Started":
                if self.on_started:
                    self.on_started()
            elif signal == "Canceled":
                if self.on_canceled:
                    self.on_canceled()
            elif signal == "Finished":
                if self.on_finished:
                    self.on_finished()

        handler_id = self.proxy.connect('g-signal', on_signal)

        source = GLib.idle_source_new()
        source.set_callback(self.started_sem.release)
        source.attach(self.thread_context)

        self.loop.run()

        self.proxy.disconnect(handler_id)
        del self.proxy


class TestPostpone(TestBase, dbusmock.DBusTestCase):
    @classmethod
    def setUpClass(cls):
        TestBase.setUpClass()
        cls.start_system_bus()
        cls.dbus_con = cls.get_dbus(system_bus=True)

    def setUp(self):
        TestBase.setUp(self)
        self.rootdir = self.make_fake_aptroot(
            template=os.path.join(self.testdir, "root.postpone"),
            fake_pkgs=[
                ("test-package", "1.0.test.pkg", {}),
            ]
        )
        self.apt_conf = os.path.join(self.rootdir, "etc", "apt", "apt.conf")
        self.stamp_file = os.path.join(self.rootdir, ".postpone-stamp")
        unattended_upgrade.POSTPONE_STAMP_FILE = self.stamp_file
        unattended_upgrade.DBUS_DIR = os.path.join(
            self.rootdir, "usr/share/dbus-1")

        # FIXME: make this more elegant
        # fake on_ac_power
        os.environ["PATH"] = (os.path.join(self.rootdir, "usr", "bin") + ":"
                              + os.environ["PATH"])

        # clean log
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")
        if not os.path.exists(os.path.dirname(self.log)):
            os.makedirs(os.path.dirname(self.log))
        with open(self.log, "w"):
            pass
        # clean cache
        subprocess.check_call(
            ["rm", "-f",
             os.path.join(self.rootdir, "var", "cache", "apt", "*.bin")])

        # clear stamp file
        os.makedirs(os.path.dirname(self.stamp_file), exist_ok=True)
        unattended_upgrade.reset_stamp_file(
            unattended_upgrade.POSTPONE_STAMP_FILE)

    def tearDown(self):
        try:
            os.remove(os.path.join(self.rootdir, "etc", "apt", "apt.conf"))
        except OSError:
            pass

    def test_postpone_days(self):
        apt.apt_pkg.config.set("Unattended-Upgrade::Postpone-For-Days", "0")
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 0)

        apt.apt_pkg.config.set("Unattended-Upgrade::Postpone-For-Days", "1")
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 1)

        subprocess.check_call(["touch", "-d", "3 days ago", self.stamp_file])
        apt.apt_pkg.config.set("Unattended-Upgrade::Postpone-For-Days", "1")
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 0)

        subprocess.check_call(["touch", "-d", "2 days ago", self.stamp_file])
        apt.apt_pkg.config.set("Unattended-Upgrade::Postpone-For-Days", "5")
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 3)

    def test_postpone_reset(self):
        subprocess.check_call(["touch", "-d", "3 days ago", self.stamp_file])
        apt.apt_pkg.config.set("Unattended-Upgrade::Postpone-For-Days", "2")
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 0)

        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=2))
        unattended_upgrade.main(
            MockOptions(), rootdir=self.rootdir)

        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 2)

    def test_no_wait_for_postpone(self):
        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=0))
        unattended_upgrade.main(
            MockOptions(), rootdir=self.rootdir)

        pkgs = unattended_upgrade.UnattendedUpgradesCache(rootdir=self.rootdir)
        self.assertFalse(pkgs["test-package"].is_upgradable)

        with open(self.log) as f:
            needle = "Waiting for a postpone request..."
            haystack = f.read()
            self.assertFalse(
                needle in haystack,
                "Should not have found '%s' in '%s'" % (needle, haystack))

    def test_wait_for_postpone(self):
        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=1))
        unattended_upgrade.main(
            MockOptions(), rootdir=self.rootdir)

        pkgs = unattended_upgrade.UnattendedUpgradesCache(rootdir=self.rootdir)
        self.assertFalse(pkgs["test-package"].is_upgradable)

        with open(self.log) as f:
            needle = "Waiting for a postpone request..."
            haystack = f.read()
            self.assertTrue(
                needle in haystack,
                "Can not find '%s' in '%s'" % (needle, haystack))

    def test_all_signals(self):
        client = PostponeClient()
        client.start()

        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=1))
        unattended_upgrade.main(
            MockOptions(), rootdir=self.rootdir)

        client.quit()
        client.join()
        self.assertEqual(
            client.received, ['AboutToStart', 'Started', 'Finished'])

    def test_signals_no_postpone(self):
        client = PostponeClient()
        client.start()

        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=0))
        unattended_upgrade.main(
            MockOptions(), rootdir=self.rootdir)

        client.quit()
        client.join()
        self.assertEqual(client.received, ['Started', 'Finished'])

    def test_do_postpone(self):
        # simulate polkit with dbusmock
        (p_mock, obj_polkit) = self.spawn_server_template(
            'polkitd', None, stdout=subprocess.PIPE)
        obj_polkit.SetAllowed(
            ["com.ubuntu.UnattendedUpgrade.Pending.Postpone"])

        client = PostponeClient()
        client.on_about_to_start = client.call_postpone
        client.start()

        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=3))
        unattended_upgrade.main(MockOptions(), rootdir=self.rootdir)

        client.quit()
        client.join()

        p_mock.stdout.close()
        p_mock.terminate()
        p_mock.wait()

        self.assertIsNone(client.error)
        self.assertEqual(client.received, ["AboutToStart", "Canceled"])
        self.assertTrue(os.path.isfile(unattended_upgrade.POSTPONE_STAMP_FILE))
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 3)

        pkgs = unattended_upgrade.UnattendedUpgradesCache(rootdir=self.rootdir)
        self.assertTrue(pkgs["test-package"].is_upgradable)

    def test_postpone_stamp_persists(self):
        apt.apt_pkg.config.set("Unattended-Upgrade::Postpone-For-Days", "3")
        subprocess.check_call(["touch", "-d", "1 days ago", self.stamp_file])
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 2)

        # simulate polkit with dbusmock
        (p_mock, obj_polkit) = self.spawn_server_template(
            'polkitd', None, stdout=subprocess.PIPE)
        obj_polkit.SetAllowed(
            ["com.ubuntu.UnattendedUpgrade.Pending.Postpone"])

        client = PostponeClient()
        client.on_about_to_start = client.call_postpone
        client.start()

        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=3))
        unattended_upgrade.main(MockOptions(), rootdir=self.rootdir)

        client.quit()
        client.join()

        p_mock.stdout.close()
        p_mock.terminate()
        p_mock.wait()

        self.assertIsNone(client.error)
        self.assertEqual(client.received, ["AboutToStart", "Canceled"])
        self.assertEqual(unattended_upgrade.get_remaining_postpone_days(), 2)

    def test_postpone_too_late(self):
        # simulate polkit with dbusmock
        (p_mock, obj_polkit) = self.spawn_server_template(
            'polkitd', None, stdout=subprocess.PIPE)
        obj_polkit.SetAllowed(
            ["com.ubuntu.UnattendedUpgrade.Pending.Postpone"])

        client = PostponeClient()
        client.on_started = client.call_postpone
        client.start()

        with open(self.apt_conf, "w") as fp:
            fp.write(APT_CONF.format(days=3))
        unattended_upgrade.main(MockOptions(), rootdir=self.rootdir)

        client.quit()
        client.join()

        p_mock.stdout.close()
        p_mock.terminate()
        p_mock.wait()

        self.assertTrue(isinstance(client.error, GLib.GError))
        self.assertTrue(
            client.error.matches(Gio.DBusError.quark(), Gio.DBusError.FAILED))
        self.assertEqual(
            client.received, ["AboutToStart", "Started", "Finished"])
        self.assertFalse(
            os.path.isfile(unattended_upgrade.POSTPONE_STAMP_FILE))

        pkgs = unattended_upgrade.UnattendedUpgradesCache(rootdir=self.rootdir)
        self.assertFalse(pkgs["test-package"].is_upgradable)


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()



#!/usr/bin/python3

import os
import unittest

from test.test_base import MockOptions, TestBase

import unattended_upgrade


class TestOnBattery(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.rootdir = self.make_fake_aptroot(
            template=os.path.join(self.testdir, "root.on-battery"))
        # FIXME: make this more elegant
        # fake on_ac_power
        os.environ["PATH"] = (os.path.join(self.rootdir, "usr", "bin") + ":"
                              + os.environ["PATH"])
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")

    def test_on_battery(self):
        # run it
        options = MockOptions()
        ret = unattended_upgrade.main(options, rootdir=self.rootdir)
        self.assertEqual(ret, 1)
        # read the log to see what happend
        with open(self.log) as f:
            needle = "System is on battery power, stopping"
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()
